package nl.logicacmg.grid.client;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeMap;

import javax.swing.ImageIcon;

import nl.logicacmg.grid.generic.Madem;
import nl.logicacmg.grid.generic.Reply;
import nl.logicacmg.grid.generic.Request;
import nl.logicacmg.grid.service.mandelbrot.MandelbrotReply;
import nl.logicacmg.grid.service.mandelbrot.MandelbrotRequest;

import org.apache.log4j.Logger;

public class MandelbrotBroker {
	Logger logger=Logger.getLogger(MandelbrotBroker.class);
	private TreeMap launchers;
	    
	public MandelbrotBroker() {
	 launchers= new TreeMap();
	 initLaunchers();
	}
		
	public void initLaunchers(){
	 	
	 Enumeration e=PropertyManager.properties.propertyNames();
     while (e.hasMoreElements()){
     	String key=(String) e.nextElement();
     	if (key.startsWith("launcher")){
     		String value=PropertyManager.properties.getProperty(key);
     		StringTokenizer st=new StringTokenizer(value,",");
     		String name=st.nextToken().trim();
     		String address=st.nextToken().trim();
     		String servicename=st.nextToken().trim();
     		boolean active=st.nextToken().trim().equals("true");
     		createLauncher(name,address,servicename,active);
     		 
     	}
     }
	}
		


	public void createLauncher(String name, String address, String servicename, boolean active) {
	 Launcher launcher= new Launcher(name, address, servicename);
	 launcher.setActive(active);
	 launchers.put(name, launcher);
	}

	private void startLaunchers() {
	 Set keys = launchers.keySet();
	 Iterator i = keys.iterator();
	 while (i.hasNext()) {
		Launcher worker = (Launcher) launchers.get(i.next());
		if (worker.isActive()) {
			worker.start();
		}
	 }
	}

	private void waitForAllLaunchers() throws InterruptedException {
	 Set keys = launchers.keySet();
	 Iterator i = keys.iterator();
	 while (i.hasNext()) {
		Launcher worker = (Launcher) launchers.get(i.next());
		if (worker.isActive()) {
		worker.join();
		}
	 }
	}


		
   public Reply broke(Request request) {
   	initLaunchers();
	split(request);
	startLaunchers();
	try {
		waitForAllLaunchers();
	} catch (InterruptedException e) {
			logger.fatal("INTERRUPTED thread in broker.perform",e);
	}
	 logger.debug("broker-combine");
	 Reply result=combine(request);
	 return result;
	}

	
	
    
	/**
	 * splits the original Request into (new) smaller requests
	 * and binds these requests to the set of workers
	 */
	public void split(Request request) {
		int n = launchers.size();
		MandelbrotRequest req = (MandelbrotRequest) request;
		int dw = req.getWidth() / n;
		double dx = (req.getMaxx() - req.getMinx()) / n;
		Set keys = launchers.keySet();
		Iterator i = keys.iterator();
		int count = 0;
		while (i.hasNext()) {
			Launcher launcher= (Launcher) launchers.get(i.next());
			if (launcher.isActive()){
			MandelbrotRequest workerrequest =
				new MandelbrotRequest(
					dw,
					req.getHeight(),
					req.getMinx() + dx * count,
					req.getMiny(),
					req.getMinx() + dx * (count + 1),
					req.getMaxy(),
					req.getIterations(),req.getColorpeaks());
			launcher.setRequest(Madem.base64(workerrequest));
			}
//					extra parameter necessary for the split and combine:
					 launcher.getParameters().add(
						 "xpos",
						 count * dw);
			
			count++;

		}
	}

	/** 
	 * combines the replies of the workers into one (new) reply
	 */
	public Reply combine(Request request) {
		MandelbrotRequest req = (MandelbrotRequest) request;
		BufferedImage image =
			new BufferedImage(req.getWidth(), req.getHeight(), BufferedImage.TYPE_INT_RGB);
	    Graphics g=image.getGraphics();
	    
		Set keys = launchers.keySet();
		Iterator i = keys.iterator();

		while (i.hasNext()) {
			Launcher launcher = (Launcher) launchers.get(i.next());
			if (launcher.isActive()) {
				MandelbrotReply workerreply = (MandelbrotReply) Madem.object(launcher.getReply());
				Image partimage = 
					((MandelbrotReply) workerreply).getImage();
				ImageIcon icon = new ImageIcon(partimage);
				int xpos=launcher.getParameters().getAsInt("xpos");
				g.drawImage(
					icon.getImage(),
					xpos,
					0,
					null);
				if (PropertyManager.getBrokerDrawInfo()){
				    drawInfo(g,launcher.getId(),xpos,0,partimage.getWidth(null)-1,partimage.getHeight(null)-1);
				}
				
			}
		}
		MandelbrotReply reply = new MandelbrotReply(req.getWidth(), req.getHeight(),image);
		return reply;
	}
	
	private void drawInfo(Graphics g,String s,int xpos,int ypos,int w,int h){
		g.setColor(Color.black);
		g.drawString(s,xpos+w/2-1,h/2-1);
		g.drawString(s,xpos+w/2-1,h/2+1);
		g.drawString(s,xpos+w/2+1,h/2-1);
		g.drawString(s,xpos+w/2+1,h/2+1);
		g.setColor(Color.white);
		g.drawString(s,xpos+w/2,h/2);
		g.drawRect(xpos,ypos,w,h);
	}
	
}