package nl.logicacmg.grid.service.mandelbrot;

import nl.logicacmg.grid.generic.Service;

import org.apache.log4j.Logger;

/**
 * G<br>enerates an image using 
 * <br>common Mandelbrot parameters.
 * <br>This class is meant be be deployed on the server/service side.  
 * <br>Optimization planned, on the level of image generation as 
 * <br>well as the encoding.
 * @author WMulder
 */
public class MandelbrotGenerator {
	private int coltable[];
	private int width;
	private int height;
	private Complex mu, z, z0;
	private int iterations;
	double minx, maxx, miny, maxy;
	double factorp, factorq;
	private int[] pixels;
	private int[][] colorpeaks;
	private final int maxcolors = 10000;
	Logger logger=Logger.getLogger(Service.class);


	public MandelbrotGenerator() {
		coltable = new int[maxcolors + 1];
    }

	public void defineColorTable(int[][] colorpeaks){
        logger.info("Mandelbrot Generator (core) builds color table...");
		for (int i = 0; i < colorpeaks.length - 1; i++) {
			definePalette(
				i * maxcolors / colorpeaks.length,
				(i + 1) * maxcolors / colorpeaks.length,
				colorpeaks[i][0],
				colorpeaks[i][1],
				colorpeaks[i][2],
				colorpeaks[i + 1][0],
				colorpeaks[i + 1][1],
				colorpeaks[i + 1][2]);
		}
	}
	
	public int[] getMandelbrotPixels(
		int width,
		int height,
		double minx,
		double miny,
		double maxx,
		double maxy,
		int iterations) {
		this.width = width;
		this.height = height;
		this.minx = minx;
		this.miny = miny;
		this.maxx = maxx;
		this.maxy = maxy;
		pixels = new int[width * height + 1];
		logger.info("Mandelbrot Generator (core) builds mandelbrot ["+minx+","+miny+","+maxx+","+maxy+"] "+width+"x"+height);
		
		z = new Complex(0, 0);
		mu = new Complex(0, 0);
		z0 = new Complex(0, 0);
		this.iterations = iterations;
		factorp = (maxx - minx) / (double) width;
		factorq = (maxy - miny) / (double) height;
		doIT();
		logger.info("Mandelbrot calculation done.");
		return pixels;
	}

	private void doIT() {
		double y = miny;
		double x;
		int t = 0;
		int row = 0;
		// corrigeren voor t die soms door afronding met factorq net eentje te
		// hoog wordt!
		while (y < maxy) {
			t = row * width;
			x = minx;
			while (x < maxx) {
				z.re = z0.re;
				z.im = z0.im;
				mu.re = x;
				mu.im = y;
				
				// julia, choose right mu...
				// z.re = x;
				// z.im = y;
				// mu.re = 0.36;
				// mu.im = 0.3;
				
				// do escape routine for this point (p,q)
				int i = 0;
				while (((z.im * z.im + z.re * z.re) < 4) && (i < iterations)) {
					Complex.msqr(z);
					Complex.madd(z, mu);
					i++;
				}
				if (t < width * height)
					pixels[t] = coltable[(i * maxcolors) / iterations];
				t++;
				x += factorp;
			}
			y += factorq;
			row++;
		}
	}
	/**
	 * colourtable creating pal - palette table[0..255] s - start e - end r1r2-
	 * red g1g2- green b1b2- blue
	 */
	private void definePalette(
		int s,
		int e,
		int r1,
		int g1,
		int b1,
		int r2,
		int g2,
		int b2) {
		int i;
		float k;
		for (i = 0; i <= e - s; i++) {
			k = (float) i / (float) (e - s);
			coltable[s + i] =
				(255 << 24)
					+ ((int) (r1 + (r2 - r1) * k) << 16)
					+ ((int) (g1 + (g2 - g1) * k) << 8)
					+ ((int) (b1 + (b2 - b1) * k));
		}
	}
}